@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 Firejail Authors
+ * Copyright (C) 2014-2018 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -24,52 +24,24 @@
 #include <dirent.h>
 #include <sys/wait.h>
 
-static void disable_file(const char *path, const char *file) {
-	assert(file);
-	assert(path);
-
-	struct stat s;
-	char *fname;
-	if (asprintf(&fname, "%s/%s", path, file) == -1)
-		errExit("asprintf");
-	if (stat(fname, &s) == -1)
-		goto doexit;
-
-	if (arg_debug)
-		printf("Disable%s\n", fname);
-
-	if (S_ISDIR(s.st_mode)) {
-		if (mount(RUN_RO_DIR, fname, "none", MS_BIND, "mode=400,gid=0") < 0)
-			errExit("disable file");
-	}
-	else {
-		if (mount(RUN_RO_FILE, fname, "none", MS_BIND, "mode=400,gid=0") < 0)
-			errExit("disable file");
-	}
-	fs_logger2("blacklist", fname);
-
-doexit:
-	free(fname);
-}
-
 // disable pulseaudio socket
 void pulseaudio_disable(void) {
 	if (arg_debug)
 		printf("disable pulseaudio\n");
 	// blacklist user config directory
-	disable_file(cfg.homedir, ".config/pulse");
+	disable_file_path(cfg.homedir, ".config/pulse");
 
 
 	// blacklist pulseaudio socket in XDG_RUNTIME_DIR
 	char *name = getenv("XDG_RUNTIME_DIR");
 	if (name)
-		disable_file(name, "pulse/native");
+		disable_file_path(name, "pulse/native");
 
 	// try the default location anyway
 	char *path;
 	if (asprintf(&path, "/run/user/%d", getuid()) == -1)
 		errExit("asprintf");
-	disable_file(path, "pulse/native");
+	disable_file_path(path, "pulse/native");
 	free(path);
 
 
@@ -87,12 +59,11 @@ void pulseaudio_disable(void) {
 	struct dirent *entry;
 	while ((entry = readdir(dir))) {
 		if (strncmp(entry->d_name, "pulse-", 6) == 0) {
-			disable_file("/tmp", entry->d_name);
+			disable_file_path("/tmp", entry->d_name);
 		}
 	}
 
 	closedir(dir);
-
 }
 
 
@@ -191,22 +162,34 @@ void pulseaudio_init(void) {
 	}
 	free(dir1);
 
-
 	// if we have ~/.config/pulse mount the new directory, else set environment variable
 	char *homeusercfg;
 	if (asprintf(&homeusercfg, "%s/.config/pulse", cfg.homedir) == -1)
 		errExit("asprintf");
 	if (stat(homeusercfg, &s) == 0) {
+		if (is_link(homeusercfg)) {
+			fprintf(stderr, "Error: user .config/pulse is a symbolic link\n");
+			exit(1);
+		}
 		if (mount(RUN_PULSE_DIR, homeusercfg, "none", MS_BIND, NULL) < 0 ||
 		    mount(NULL, homeusercfg, NULL, MS_NOEXEC|MS_NODEV|MS_NOSUID|MS_BIND|MS_REMOUNT, NULL) < 0)
 			errExit("mount pulseaudio");
 		fs_logger2("tmpfs", homeusercfg);
+
+		// check /proc/self/mountinfo to confirm the mount is ok
+		MountData *mptr = get_last_mount();
+		if (strcmp(mptr->dir, homeusercfg) != 0)
+			errLogExit("invalid pulseaudio mount");
+		if (strcmp(mptr->fstype, "tmpfs") != 0)
+			errLogExit("invalid pulseaudio mount");
+
 		char *p;
 		if (asprintf(&p, "%s/client.conf", homeusercfg) == -1)
 			errExit("asprintf");
 		fs_logger2("create", p);
 		free(p);
 	}
+
 	else {
 		// set environment
 		if (setenv("PULSE_CLIENTCONFIG", pulsecfg, 1) < 0)
