@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 Firejail Authors
+ * Copyright (C) 2014-2018 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -103,5 +103,5 @@ void shut(pid_t pid) {
 		}
 	}
 
-	clear_run_files(parent);
+	delete_run_files(parent);
 }
