@@ -1,6 +1,6 @@
 #!/bin/bash
 # This file is part of Firejail project
-# Copyright (C) 2014-2017 Firejail Authors
+# Copyright (C) 2014-2018 Firejail Authors
 # License GPL v2
 
 export MALLOC_CHECK_=3
@@ -10,20 +10,20 @@ echo "TESTING: no x11 (test/apps-x11/x11-none.exp)"
 ./x11-none.exp
 
 
-which xterm
+which xterm 2>/dev/null
 if [ "$?" -eq 0 ];
 then
 	echo "TESTING: xterm x11 xorg"
 	./xterm-xorg.exp
 
-	which xpra
+	which xpra 2>/dev/null
 	if [ "$?" -eq 0 ];
 	then
 	echo "TESTING: xterm x11 xpra"
 	./xterm-xpra.exp
 	fi
 
-	which Xephyr
+	which Xephyr 2>/dev/null
 	if [ "$?" -eq 0 ];
 	then
 	echo "TESTING: xterm x11 xephyr"
@@ -34,13 +34,13 @@ else
 fi
 
 # check xpra/xephyr
-which xpra
+which xpra 2>/dev/null
 if [ "$?" -eq 0 ];
 then
         echo "xpra found"
 else
         echo "xpra not found"
-	which Xephyr
+	which Xephyr 2>/dev/null
 	if [ "$?" -eq 0 ];
 	then
         	echo "Xephyr found"
@@ -50,7 +50,7 @@ else
 	fi
 fi
 
-which firefox
+which firefox 2>/dev/null
 if [ "$?" -eq 0 ];
 then
 	echo "TESTING: firefox x11"
@@ -59,7 +59,7 @@ else
 	echo "TESTING SKIP: firefox not found"
 fi
 
-which chromium
+which chromium 2>/dev/null
 if [ "$?" -eq 0 ];
 then
 	echo "TESTING: chromium x11"
@@ -68,7 +68,7 @@ else
 	echo "TESTING SKIP: chromium not found"
 fi
 
-which transmission-gtk
+which transmission-gtk 2>/dev/null
 if [ "$?" -eq 0 ];
 then
 	echo "TESTING: transmission-gtk x11"
@@ -77,7 +77,7 @@ else
 	echo "TESTING SKIP: transmission-gtk not found"
 fi
 
-which thunderbird
+which thunderbird 2>/dev/null
 if [ "$?" -eq 0 ];
 then
 	echo "TESTING: thunderbird x11"
