@@ -1,21 +1,45 @@
 #!/bin/bash
 # This file is part of Firejail project
-# Copyright (C) 2014-2017 Firejail Authors
+# Copyright (C) 2014-2018 Firejail Authors
 # License GPL v2
 
 export MALLOC_CHECK_=3
 export MALLOC_PERTURB_=$(($RANDOM % 255 + 1))
 
+if [ -f /etc/debian_version ]; then
+	libdir=$(dirname "$(dpkg -L firejail | grep faudit)")
+	export PATH="$PATH:$libdir"
+fi
+export PATH="$PATH:/usr/lib/firejail:/usr/lib64/firejail"
+
+echo "testing" > ~/firejail-test-file-7699
+echo "testing" > /tmp/firejail-test-file-7699
+echo "testing" > /var/tmp/firejail-test-file-7699
+echo "TESTING: build (test/utils/build.exp)"
+./build.exp
+rm -f ~/firejail-test-file-7699
+rm -f /tmp/firejail-test-file-7699
+rm -f /var/tmp/firejail-test-file-7699
+
 echo "TESTING: audit (test/utils/audit.exp)"
 ./audit.exp
 
+echo "TESTING: name (test/utils/name.exp)"
+./name.exp
+
+echo "TESTING: command (test/utils/command.exp)"
+./command.exp
+
+echo "TESTING: profile.print (test/utils/profile_print.exp)"
+./profile_print.exp
+
 echo "TESTING: version (test/utils/version.exp)"
 ./version.exp
 
 echo "TESTING: help (test/utils/help.exp)"
 ./help.exp
 
-which man
+which man 2>/dev/null
 if [ "$?" -eq 0 ];
 then
         echo "TESTING: man (test/utils/man.exp)"
